﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;

namespace WindowsGame1.GameObjects
{
    class MegaExplosion : GameObject
    {
        public delegate void MegaExplosionCompleteHandler();

        public static void Perform(GameLevel gl, Point p, MegaExplosionCompleteHandler h)
        {
            MegaExplosion ex = new MegaExplosion();

            gl.AlternateTickProcess = delegate
            {
                ex.Tick();
                foreach (GameObject obj in ex.AnimationItems)
                {
                    if (obj != null && obj.GameLevel == gl)
                        obj.Tick();
                }
            };
            ex.Location = p;
            ex.ExplosionComplete = delegate
            {
                gl.AlternateTickProcess = null;
                h();
            };
            gl.GameObjects.Add(ex);
        }

        public MegaExplosion()
        {
            this.BoundingBox = new Rectangle(0, 0, 8, 8);
            this.TickPriority = TickPriority.Environmental;
        }

        public Point Location
        {
            get { return this.BoundingBox.Center; }
            set
            {
                this.BoundingBox = new Rectangle(value.X - (this.BoundingBox.Width / 2), value.Y - (this.BoundingBox.Height / 2),
                    this.BoundingBox.Width, this.BoundingBox.Height);
            }
        }

        private long ElapsedTickCount = 0;

        private const int MiniSpinSteps = 8;
        private const int MiniSpinStepLengthTicks = 2;
        private const int ShrapnelPhaseLengthTicks = 60 * 4;

        private enum AnimationPhase
        {
            None,
            MiniSpin,
            Shrapnel
        }

        private AnimationPhase m_currentAnimationPhase = AnimationPhase.None;

        private AnimationPhase CurrentAnimationPhase
        {
            get { return m_currentAnimationPhase; }
            set
            {
                if (value != m_currentAnimationPhase)
                {
                    TeardownAnimationPhase(m_currentAnimationPhase);
                    m_currentAnimationPhase = value;
                    SetupAnimationPhase(m_currentAnimationPhase);
                }
            }
        }

        private void TeardownAnimationPhase(AnimationPhase phase)
        {
            foreach (GameObject obj in AnimationItems)
            {
                if (obj != null && obj.GameLevel == this.GameLevel)
                    this.GameLevel.GameObjects.Remove(obj);
            }
            AnimationItems.Clear();
        }

        private void SetupAnimationPhase(AnimationPhase phase)
        {
            switch (phase)
            {
                case AnimationPhase.MiniSpin:
                    for (int i = 0; i < 2; i++)
                    {
                        MiniSpinParticle msp = new MiniSpinParticle();
                        AnimationItems.Add(msp);
                        this.GameLevel.GameObjects.Add(msp);
                    }
                    this.GameLevel.SoundEffects["Player.SFX.Death"].Play();
                    break;
                case AnimationPhase.Shrapnel:
                    KeyValuePair<Vector2, float>[] vecs = new KeyValuePair<Vector2, float>[] {
                        new KeyValuePair<Vector2, float>(new Vector2(0.0f, -1.0f), 2.0f),
                        new KeyValuePair<Vector2, float>(new Vector2(1.0f, -1.0f), 2.0f),
                        new KeyValuePair<Vector2, float>(new Vector2(1.0f, 0.0f), 2.0f),
                        new KeyValuePair<Vector2, float>(new Vector2(1.0f, 1.0f), 2.0f),
                        new KeyValuePair<Vector2, float>(new Vector2(0.0f, 1.0f), 2.0f),
                        new KeyValuePair<Vector2, float>(new Vector2(-1.0f, 1.0f), 2.0f),
                        new KeyValuePair<Vector2, float>(new Vector2(-1.0f, 0.0f), 2.0f),
                        new KeyValuePair<Vector2, float>(new Vector2(-1.0f, -1.0f), 2.0f),
                        new KeyValuePair<Vector2, float>(new Vector2(0.0f, -1.0f), 1.0f),
                        new KeyValuePair<Vector2, float>(new Vector2(1.0f, 0.0f), 1.0f),
                        new KeyValuePair<Vector2, float>(new Vector2(0.0f, 1.0f), 1.0f),
                        new KeyValuePair<Vector2, float>(new Vector2(-1.0f, 0.0f), 1.0f)
                    };
                    for (int i = 0; i < 12; i++)
                    {
                        ShrapnelParticle sp = new ShrapnelParticle();
                        sp.Location = this.Location;
                        Vector2 vec = vecs[i].Key;
                        vec.Normalize();
                        sp.Vector = vec;
                        sp.VectorScale = vecs[i].Value;
                        AnimationItems.Add(sp);
                        this.GameLevel.GameObjects.Add(sp);
                    }
                    break;
                case AnimationPhase.None:
                default:
                    break;
            }
        }

        private class MegaExplosionParticle : GameObject
        {
            public MegaExplosionParticle(int x, int y)
            {
                this.BoundingBox = new Rectangle(0, 0, x, y);
                this.TickPriority = TickPriority.None;
            }

            public Point Location
            {
                get { return this.BoundingBox.Center; }
                set
                {
                    this.BoundingBox = new Rectangle(value.X - (this.BoundingBox.Width / 2), value.Y - (this.BoundingBox.Height / 2),
                        this.BoundingBox.Width, this.BoundingBox.Height);
                }
            }
        }

        private class MiniSpinParticle : MegaExplosionParticle
        {
            public MiniSpinParticle()
                : base(12, 12)
            {
                this.TextureName = "Explosions.Explode.2";
            }
        }

        private class ShrapnelParticle : MegaExplosionParticle
        {
            public ShrapnelParticle()
                : base(16, 16)
            {
                this.TextureName = AnimationFrames[0];
                this.TickPriority = TickPriority.Environmental;
            }

            private const int AnimationFrameLength = 2;
            private static readonly string[] AnimationFrames = new string[] {
                "Explosions.Explode.1",
                "Explosions.Explode.2",
                "Explosions.Explode.3",
                "Explosions.Explode.4",
                null
            };

            public Vector2 Vector = new Vector2(0.0f, 0.0f);
            public float VectorScale = 1.0f;

            private long ElapsedTickCount = 0;

            internal override void Tick()
            {
                this.TextureName = AnimationFrames[(ElapsedTickCount / AnimationFrameLength) % AnimationFrames.Length];
                ElapsedTickCount++;
            }

            internal override void Draw(Point point, SpriteBatch spriteBatch)
            {
                if (this.TextureName != null)
                {
                    Texture2D tex = this.GameLevel.Textures[this.TextureName];
                    spriteBatch.Draw(tex, new Rectangle(this.Location.X - (tex.Width / 2) - point.X, this.Location.Y - (tex.Height / 2) - point.Y,
                        tex.Width, tex.Height),
                        Color.White);
                }
            }
        }

        private List<MegaExplosionParticle> AnimationItems = new List<MegaExplosionParticle>(12);

        internal override void Tick()
        {
            if (ElapsedTickCount < (MiniSpinSteps * MiniSpinStepLengthTicks))
            {
                // Do mini spin
                this.CurrentAnimationPhase = AnimationPhase.MiniSpin;
                switch ((ElapsedTickCount / MiniSpinStepLengthTicks) % 4)
                {
                    default:
                    case 0:
                        AnimationItems[0].Location = new Point(this.Location.X, this.Location.Y - 7);
                        AnimationItems[1].Location = new Point(this.Location.X, this.Location.Y + 7);
                        break;
                    case 1:
                        AnimationItems[0].Location = new Point(this.Location.X - 3, this.Location.Y - 3);
                        AnimationItems[1].Location = new Point(this.Location.X + 3, this.Location.Y + 3);
                        break;
                    case 2:
                        AnimationItems[0].Location = new Point(this.Location.X - 7, this.Location.Y);
                        AnimationItems[1].Location = new Point(this.Location.X + 7, this.Location.Y);
                        break;
                    case 3:
                        AnimationItems[0].Location = new Point(this.Location.X - 3, this.Location.Y + 3);
                        AnimationItems[1].Location = new Point(this.Location.X + 3, this.Location.Y - 3);
                        break;
                }
            }
            else if (ElapsedTickCount < (MiniSpinSteps * MiniSpinStepLengthTicks) + ShrapnelPhaseLengthTicks)
            {
                if (this.CurrentAnimationPhase != AnimationPhase.Shrapnel)
                    OnMiniExplosionComplete();

                // Do shrapnel expansion
                this.CurrentAnimationPhase = AnimationPhase.Shrapnel;

                float dist = ElapsedTickCount - (MiniSpinSteps * MiniSpinStepLengthTicks);
                for (int i = 0; i < 12; i++)
                {
                    Vector2 v = ((ShrapnelParticle)AnimationItems[i]).Vector;
                    float vs = ((ShrapnelParticle)AnimationItems[i]).VectorScale;
                    v = v * vs * dist;
                    AnimationItems[i].Location = new Point(this.Location.X + Convert.ToInt32(v.X), this.Location.Y + Convert.ToInt32(v.Y));
                }
            }
            else
            {
                this.CurrentAnimationPhase = AnimationPhase.None;
                OnExplosionComplete();
                this.GameLevel.GameObjects.Remove(this);
            }

            ElapsedTickCount++;

            base.Tick();
        }

        internal override void Draw(Point point, SpriteBatch spriteBatch)
        {
            // Nothing to draw.
        }

        private void OnMiniExplosionComplete()
        {
            MegaExplosionCompleteHandler temp = MiniExplosionComplete;
            if (temp != null)
            {
                temp();
            }
        }

        private void OnExplosionComplete()
        {
            MegaExplosionCompleteHandler temp = ExplosionComplete;
            if (temp != null)
            {
                temp();
            }
        }

        public event MegaExplosionCompleteHandler MiniExplosionComplete;
        public event MegaExplosionCompleteHandler ExplosionComplete;
    }
}
